@@ -2,6 +2,7 @@ import aiohttp
 import logging
 import time
 import requests
+from datetime import datetime
 from hashlib import sha256
 from ecdsa import SigningKey
 
@@ -103,9 +104,9 @@ class XenaMDClient(XenaClient):
         :param timeframe: timeframe of bars
         :type timeframe: str '1m', '15m' '30m', '1h', '3h', '6h', '24h'
         :param ts_from: show candles from
-        :type ts_from: int unixtimestamp
+        :type ts_from: int unixtimestamp in nanoseconds
         :param ts_from: show candles to
-        :type ts_to: int unixtimestamp
+        :type ts_to: int unixtimestamp in nanoseconds
 
         :returns: xena.proto.market_pb2.MarketDataRefresh
         """
@@ -115,21 +116,50 @@ class XenaMDClient(XenaClient):
             "to": ts_to,
         })
     
-    async def dom(self, symbol, aggr=0):
+    async def dom(self, symbol):
         """Get L2 snapshot for :symbol 
 
         :param symbol: required
         :type symbol: str
-        :param aggr: aggregate leveles 
-        :type timeframe: int [0,5,10,25,50,100,250]
 
         :returns: xena.proto.market_pb2.MarketDataRefresh
         """
 
-        return await self._get('/market-data/dom/'+symbol, msg=market_pb2.MarketDataRefresh, params={
-            "aggr": aggr
+        return await self._get('/market-data/dom/'+symbol, msg=market_pb2.MarketDataRefresh)
+    
+    async def trades(self, symbol, ts_from="", ts_to="", page=1, limit=0):
+        """Get trades for :symbol 
+
+        :param symbol: required
+        :type symbol: str
+        :param ts_from: show trades which TransactTime greater than ts_from, max value is now - 7 days
+        :type ts_from: int unixtimestamp in nanoseconds
+        :param ts_to: show trades which TransactTime less than ts_to
+        :type ts_to: int unixtimestamp in nanoseconds
+        :param page: use for pagination to move through pages
+        :type page: int
+        :param limit: number of trades to return
+        :type limit: int
+
+        :returns: xena.proto.market_pb2.MarketDataRefresh
+        """
+
+        return await self._get('/market-data/trades/'+symbol, msg=market_pb2.MarketDataRefresh, params={
+            "from": ts_from,
+            "to": ts_to,
+            "page": page,
+            "limit": limit
         })
     
+    async def server_time(self):
+        """Get server time
+        
+        :returns: datetime.datetime
+        """ 
+
+        resp = await self._get('/market-data/server-time', msg=common_pb2.Heartbeat)
+        return datetime.fromtimestamp(resp.TransactTime/1000000000)
+    
     async def instruments(self):
         """Get list of instruments
         
@@ -155,11 +185,21 @@ class XenaMDSyncClient(XenaSyncClient):
             "to": ts_to,
         })
     
-    def dom(self, symbol, aggr=0):
-        return self._get('/market-data/dom/'+symbol, msg=market_pb2.MarketDataRefresh, params={
-            "aggr": aggr
+    def dom(self, symbol):
+        return self._get('/market-data/dom/'+symbol, msg=market_pb2.MarketDataRefresh)
+    
+    def trades(self, symbol, ts_from="", ts_to="", page=1, limit=0):
+        return self._get('/market-data/trades/'+symbol, msg=market_pb2.MarketDataRefresh, params={
+            "from": ts_from,
+            "to": ts_to,
+            "page": page,
+            "limit": limit
         })
     
+    def server_time(self):
+        resp = self._get('/market-data/server-time', msg=common_pb2.Heartbeat)
+        return datetime.fromtimestamp(resp.TransactTime/1000000000)
+    
     def instruments(self):
         return self._get('/public/instruments', msg=common_pb2.Instrument)
 
@@ -246,6 +286,20 @@ class XenaTradingClient(XenaClient):
         cmd.Account = account
         return await self.cancel(cmd)
 
+    async def masss_cancel(self, account, cancel_id, symbol="", side="", position_effect=constants.PositionEffect_Default):
+        """ Create OrderMassCancelRequest and send request"""
+
+        cmd = order_pb2.OrderMassCancelRequest
+        cmd.MsgType = constants.MsgType_OrderMassCancelRequest
+        cmd.MassCancelRequestType = constants.MassCancelRequestType_CancelOrdersForASecurity if symbol != "" else constants.MassCancelRequestType_CancelAllOrders
+        cmd.ClOrdId = cancel_id
+        cmd.Account = account
+        cmd.Symbol = symbol
+        cmd.Side = side
+        cmd.PositionEffect = position_effect
+        cmd.TransactTime = int(time.time() * 1000000000)
+        return await self._post('/order/mass-cancel', data=serialization.to_json(cmd))
+
     async def replace(self, cmd):
         """Wrapper for post with convenient methond name for sending replace commnads"""
 
@@ -277,6 +331,43 @@ class XenaTradingClient(XenaClient):
 
         return await self._get('/accounts/' + str(account) + '/positions')
     
+    async def positions_for_symbol(self, account, symbol):
+        """Request list of open positions for :account and :symbol
+        
+        :param account: required
+        :type account: int
+        :param symbol: required
+        :type account: string
+        :returns: list of xena.proto.positions_pb2.PositionReport
+        """
+
+        result = []
+        for p in await self.positions(account):
+            if p.Symbol == symbol:
+                result.append(p)
+
+        return result
+    
+    async def aggregate_positions_volume(self, account, symbol):
+        """Request list of open positions for :account and :symbol
+        
+        :param account: required
+        :type account: int
+        :param symbol: required
+        :type account: string
+        :returns: list of xena.proto.positions_pb2.PositionReport
+        """
+
+        positions = await self.positions_for_symbol(account, symbol)
+        if len(positions) == 0:
+            return "0"
+
+        result = helpers.aggregate_positions_volume(positions)
+        if symbol not in result:
+            return "0"
+        
+        return result[symbol]
+    
     async def positions_history(self, account, id=0, parentid=0, symbol="", open_ts_from=0, open_ts_to=0, close_ts_from=0, close_ts_to=0, page=1, limit=0):
         """Request position history for :account
         
@@ -289,13 +380,13 @@ class XenaTradingClient(XenaClient):
         :param symbol: filter positions by symbol
         :type sybmol: string
         :param open_ts_from: show positions which PositionOpenTime greater than open_ts_from
-        :type open_ts_from: int unixtimestamp
+        :type open_ts_from: int unixtimestamp in nanoseconds
         :param open_ts_from: show trades which PositionOpenTime less than open_ts_to
-        :type open_ts_to: int unixtimestamp
+        :type open_ts_to: int unixtimestamp in nanoseconds
         :param close_ts_from: show positions which SettlDate greater than close_ts_from
-        :type close_ts_from: int unixtimestamp
+        :type close_ts_from: int unixtimestamp in nanoseconds
         :param close_ts_from: show trades which SettlDate less than close_ts_to
-        :type close_ts_to: int unixtimestamp
+        :type close_ts_to: int unixtimestamp in nanoseconds
         :param page: use for pagination to move through pages
         :type page: int
         :param limit: number of position to return
@@ -337,9 +428,9 @@ class XenaTradingClient(XenaClient):
         :param symbol: filter trades by symbol
         :type sybmol: string
         :param ts_from: show trades which TransactTime greater than ts_from
-        :type ts_from: int unixtimestamp
-        :param ts_from: show trades which TransactTime less than ts_to
-        :type ts_to: int unixtimestamp
+        :type ts_from: int unixtimestamp in nanoseconds
+        :param ts_to: show trades which TransactTime less than ts_to
+        :type ts_to: int unixtimestamp in nanoseconds
         :param page: use for pagination to move through pages
         :type page: int
         :param limit: number of trades to return
@@ -462,6 +553,18 @@ class XenaTradingSyncClient(XenaSyncClient):
         cmd.Account = account
         return self.cancel(cmd)
 
+    def masss_cancel(self, account, cancel_id, symbol="", side="", position_effect=constants.PositionEffect_Default):
+        cmd = order_pb2.OrderMassCancelRequest
+        cmd.MsgType = constants.MsgType_OrderMassCancelRequest
+        cmd.MassCancelRequestType = constants.MassCancelRequestType_CancelOrdersForASecurity if symbol != "" else constants.MassCancelRequestType_CancelAllOrders
+        cmd.ClOrdId = cancel_id
+        cmd.Account = account
+        cmd.Symbol = symbol
+        cmd.Side = side
+        cmd.PositionEffect = position_effect
+        cmd.TransactTime = int(time.time() * 1000000000)
+        return self._post('/order/mass-cancel', data=serialization.to_json(cmd))
+
     def replace(self, cmd):
         if not isinstance(cmd, order_pb2.OrderCancelReplaceRequest):
             raise ValueError("Command has to be OrderCancelRequest")
@@ -482,6 +585,25 @@ class XenaTradingSyncClient(XenaSyncClient):
     def positions(self, account):
         return self._get('/accounts/' + str(account) + '/positions')
     
+    def positions_for_symbol(self, account, symbol):
+        result = []
+        for p in self.positions(account):
+            if p.Symbol == symbol:
+                result.append(p)
+
+        return result
+    
+    def aggregate_positions_volume(self, account, symbol):
+        positions = self.positions_for_symbol(account, symbol)
+        if len(positions) == 0:
+            return "0"
+
+        result = helpers.aggregate_positions_volume(positions)
+        if symbol not in result:
+            return "0"
+        
+        return result[symbol]
+    
     def positions_history(self, account, id=0, parentid=0, symbol="", open_ts_from=0, open_ts_to=0, close_ts_from=0, close_ts_to=0, page=1, limit=0):
         return self._get('/accounts/' + str(account) + '/positions-history', params={
             "id": id,
